import mitt from 'mitt';

function Store (initialState) {
  var _state = initialState;
  this.actions = {};
  this.events = mitt();
  this.on = this.events.on;
  this.off = this.events.off;
  this.emit = this.emit.bind(this);
  this.handleActions = this.handleActions.bind(this);

  Object.defineProperty(this, 'state', {
    get: function () {
      return _state
    },
    set: function (value) {
      _state = value;
    }
  });
}

Object.assign(Store.prototype, {
  emit: function emit (type, payload) {
    if (typeof type === 'function') { return type(this) }
    this.events.emit(type, payload);
  },

  createActions: function createActions (actionMap) {
    var this$1 = this;

    var loop = function ( creatorName ) {
      var type = actionMap[creatorName];
      var actionCreator = (void 0);
      if (typeof type === 'function') {
        actionCreator = function (payload) { return this$1.emit(type(payload)); };
      } else {
        actionCreator = function (payload) { return this$1.emit(type, payload); };
        actionCreator.toString = function () { return type.toString(); };
      }
      this$1.actions[creatorName] = actionCreator;
    };

    for (var creatorName in actionMap) loop( creatorName );
  },

  handleActions: function handleActions (handlerMap) {
    var this$1 = this;

    var loop = function ( type ) {
      var handler = function (eventType, e) {
        if (eventType.substring(0, 8) === '$$store:') { return }
        this$1.state = handlerMap[type](this$1.state, e, eventType) || this$1.state;
        this$1.events.emit('$$store:state:change', this$1.state);
      };
      if (type !== '*') {
        handler = handler.bind(null, type);
      }
      this$1.events.on(type, handler);
    };

    for (var type in handlerMap) loop( type );
  }
});

function createStore (initialState) {
  return new Store(initialState)
}

export { createStore };
//# sourceMappingURL=smitty.es.js.map
